@@ -12,7 +12,6 @@
  *  any later version.
  *
  *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
@@ -92,7 +91,7 @@ mb_wm_client_base_destroy (MBWMObject *this)
 
   wm = client->wmref;
 
-  mb_wm_util_trap_x_errors();
+  mb_wm_util_async_trap_x_errors(wm->xdpy);
 
   if (client->xwin_frame)
     {
@@ -111,8 +110,7 @@ mb_wm_client_base_destroy (MBWMObject *this)
       client->xwin_modal_blocker = None;
     }
 
-  XSync(wm->xdpy, False);
-  mb_wm_util_untrap_x_errors();
+  mb_wm_util_async_untrap_x_errors();
 
   parent = mb_wm_client_get_transient_for (MB_WM_CLIENT(this));
   if (parent)
@@ -261,7 +259,7 @@ mb_wm_client_base_realize (MBWindowManagerClient *client)
                       MB_WM_CLIENT_XWIN(client),
                       client->wmref->root_win->xwindow, 0, 0);
     }
-      
+
   /*
    * If this is a system-modal client and the global setting
    * is to support system modal windows, we create a
@@ -303,14 +301,14 @@ mb_wm_client_base_realize (MBWindowManagerClient *client)
     }
 
   XSetWindowBorderWidth(wm->xdpy, MB_WM_CLIENT_XWIN(client), 0);
-  
+
   XSelectInput(wm->xdpy,
 	       MB_WM_CLIENT_XWIN(client),
 	       PropertyChangeMask);
   mb_wm_client_window_sync_properties (
-		  client->window, 
+		  client->window,
 		  MBWM_WINDOW_PROP_TRANSIENCY);
-  
+
 
   XAddToSaveSet(wm->xdpy, MB_WM_CLIENT_XWIN(client));
 }
@@ -353,7 +351,7 @@ mb_wm_client_base_stack (MBWindowManagerClient *client,
   /*
    * And then we are going to use the parent.
    */
-  if (transient_for) 
+  if (transient_for)
 	  client = transient_for;
 
   /*
@@ -588,7 +586,7 @@ mb_wm_client_base_display_sync (MBWindowManagerClient *client)
 		}
 	      else
 		{
-		  if (is_window_mapped(wm->xdpy, MB_WM_CLIENT_XWIN(client))) 
+		  if (is_window_mapped(wm->xdpy, MB_WM_CLIENT_XWIN(client)))
                     {
                       client->skip_unmaps++;
                       MB_WM_DBG_SKIP_UNMAPS (client);
@@ -598,7 +596,7 @@ mb_wm_client_base_display_sync (MBWindowManagerClient *client)
 				  wm->root_win->xwindow, 0, 0);
 		  XUnmapWindow(wm->xdpy, client->xwin_frame);
 		  XMapWindow(wm->xdpy, MB_WM_CLIENT_XWIN(client));
-		  
+
 		}
 	    }
 	  else if (!client->window->undecorated)
@@ -615,8 +613,8 @@ mb_wm_client_base_display_sync (MBWindowManagerClient *client)
 	}
 
       mb_wm_client_request_geometry (
-	  client, 
-	  &client->frame_geometry, 
+	  client,
+	  &client->frame_geometry,
 	  MBWMClientReqGeomForced);
 
       if (wm->focused_client == client)
@@ -638,7 +636,7 @@ mb_wm_client_base_display_sync (MBWindowManagerClient *client)
       int x, y, w, h;
       CARD32 wgeom[4];
 
-      mb_wm_util_trap_x_errors();
+      mb_wm_util_async_trap_x_errors(wm->xdpy);
 
       if (fullscreen || !client->xwin_frame)
 	{
@@ -688,14 +686,13 @@ mb_wm_client_base_display_sync (MBWindowManagerClient *client)
 		      XA_CARDINAL, 32, PropModeReplace,
 		      (unsigned char *)&wgeom[0], 4);
 
+      mb_wm_util_async_untrap_x_errors();
       /* FIXME: need flags to handle other stuff like configure events etc */
 
       /* Resize any decor */
       mb_wm_util_list_foreach(client->decor,
 			      (MBWMListForEachCB)mb_wm_decor_handle_resize,
 			      NULL);
-
-      mb_wm_util_untrap_x_errors();
     }
   else if (mb_wm_client_needs_configure_request_ack (client))
     send_synthetic_configure_notify (client);
@@ -705,7 +702,7 @@ mb_wm_client_base_display_sync (MBWindowManagerClient *client)
 
   if (mb_wm_client_needs_visibility_sync (client))
     {
-      mb_wm_util_trap_x_errors();
+      mb_wm_util_async_trap_x_errors(wm->xdpy);
 
       if (mb_wm_client_is_mapped (client))
 	{
@@ -746,17 +743,15 @@ mb_wm_client_base_display_sync (MBWindowManagerClient *client)
 	}
 
       mb_wm_client_base_set_state_props (client);
-      mb_wm_util_untrap_x_errors();
+      mb_wm_util_async_untrap_x_errors();
     }
 
   /* Paint any decor */
-
-  mb_wm_util_trap_x_errors();
-
   if (mb_wm_client_needs_decor_sync (client))
     {
       MBGeometry area;
 
+      mb_wm_util_async_trap_x_errors(wm->xdpy);
       /*
        * First, we set the base shape mask, if needed, so that individual
        * decors can add themselves to it.
@@ -798,16 +793,14 @@ mb_wm_client_base_display_sync (MBWindowManagerClient *client)
 	    XMapWindow(wm->xdpy, client->xwin_frame);
 	}
 #endif
+      mb_wm_util_async_untrap_x_errors();
+
       mb_wm_util_list_foreach (client->decor,
 			      (MBWMListForEachCB)mb_wm_decor_handle_repaint,
 			      NULL);
     }
 
-  mb_wm_util_untrap_x_errors();
 
-  mb_wm_util_trap_x_errors();
-  XSync(wm->xdpy, False);
-  mb_wm_util_untrap_x_errors();
 }
 
 /* Note request geometry always called by layout manager */
